#include <stdio.h>
/* srand, strtol */
#include <stdlib.h>
/* time */
#include <time.h>
/* strcmp */
#include <string.h>

#include "array.h"
#include "build.h"
#include "grid-detail.h"
#include "util-grid.h"
#include "util.h"
#include "cave.h"

int
main(int argc, char *argv[])
{
  int i;
  int status;
  int build_plan;
  int size_x;
  int size_y;
  int **grid = NULL;

  build_plan = 0;
  size_x = 75;
  size_y = 75;

  for (i = 1; i < argc; i++)
  {
    if (!strcmp(argv[i], "--plan"))
    {
      if (i + 1 >= argc)
      {
        fprintf(stderr, "main: --plan takes 1 argument\n");
        return 1;
      }
      if (str_to_int(&build_plan, argv[i + 1], 0) != 0)
      {
        fprintf(stderr, "main: str_to_int(build_plan) failed\n");
        return 1;
      }
      i++;
      continue;
    }
    else if (!strcmp(argv[i], "--size"))
    {
      if (i + 2 >= argc)
      {
        fprintf(stderr, "main: --size takes 2 arguments\n");
        return 1;
      }
      if (str_to_int(&size_x, argv[i + 1], 0) != 0)
      {
        fprintf(stderr, "main: str_to_int(size_x) failed\n");
        return 1;
      }
      if (str_to_int(&size_y, argv[i + 2], 0) != 0)
      {
        fprintf(stderr, "main: str_to_int(size_x) failed\n");
        return 1;
      }
      i += 2;
      continue;
    }
    else
    {
      fprintf(stderr, "main: unknown option %s\n", argv[i]);
      return 1;
    }
  }

  if (size_x <= 0)
  {
    fprintf(stderr, "main: size_x must be positive (%d)\n", size_x);
    return 1;
  }
  if (size_y <= 0)
  {
    fprintf(stderr, "main: size_y must be positive (%d)\n", size_y);
    return 1;
  }

  grid = array2_new(size_x, size_y);
  if (grid == NULL)
  {
    fprintf(stderr, "main: array2_new failed\n");
    return 1;
  }

  srand((unsigned int) time(NULL));

  switch (build_plan)
  {
  case 0:
    /* some rooms and long corridors */
    status = build_r_and_c(grid, size_x, size_y,
                           1, 1,
                           4, 5,
                           1, CONNECT_SOME);
    break;
  case 1:
    /* many small rooms and short corridors */
    status = build_r_and_c(grid, size_x, size_y,
                           1, 1,
                           4, 0,
                           2, CONNECT_SOME);
    break;
  case 2:
    /* some rooms and wide corridors */
    status = build_r_and_c(grid, size_x, size_y,
                           2, 1,
                           3, 1,
                           1, CONNECT_SOME);
    break;
  case 3:
    /* a standard maze */
    status = build_r_and_c(grid, size_x, size_y,
                           1, 2,
                           1, 0,
                           0, CONNECT_MIN);
    break;
  case 4:
    /* walls with (almost) no dead end */
    status = build_r_and_c(grid, size_x, size_y,
                           2, 1,
                           1, 0,
                           0, CONNECT_DEAD_END);
    break;
  case 5:
    /* a "braid" maze */
    status = build_braid(grid, size_x, size_y, 3);
    break;
  case 6:
    /* a hardcore "braid" maze */
    status = build_braid(grid, size_x, size_y, 0);
    break;
  case 7:
    /* a cave hall */
    status = build_cave(grid, size_x, size_y, 45);
    break;
  case 8:
    /* a winding cave */
    status = build_cave(grid, size_x, size_y, 25);
    break;
  default:
    fprintf(stderr, "main: strange build_plan (%d)\n", build_plan);
    array2_delete(grid, size_x, size_y);
    grid = NULL;
    return 1;
    break;
  }

  if (status != 0)
  {
    fprintf(stderr, "main: build failed\n");
    /* doing dump_grid() anyway may be useful for debugging */
  }

  if (status == 0)
  {    
    if (check_connection(grid, size_x, size_y) != 0)
    {
      fprintf(stderr, "main: check_connection failed\n");
    }
  }

  if (status == 0)
  {    
    dump_grid(grid, size_x, size_y);
  }

  array2_delete(grid, size_x, size_y);
  grid = NULL;

  return 0;
}
